<?php
namespace PME\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\HTTP\Request;

class CrudPME extends \PME\Services\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	// protected $crudNext;
	protected $errorCode = [
		"ORA-02292" => "ERROR_REGISTROFILHO"
	];

    // public function __construct($crudService, $parameterBag, $crudNext) {
    public function __construct($crudService, $parameterBag) {
		parent::__construct($crudService, $parameterBag);
		// $this->crudNext = $crudNext;
	}

	
	public function save(DTO\Request $request, DTO\Response $response) {
		try {	
			$params = $request->getParameters();
			$rows = [];
			$this->dataSourceName = $this->getDataSourceName($params);
			
			foreach($params['editedRows'] as $row){
			    array_push($rows, $row);
			}
			$dataSet = new DataSet($this->dataSourceName, $rows);
	        $request = new DTO\Request\DataSet($dataSet, "", "", "");  
			$rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);  
	        $this->crudService->save($this->dataSourceName, $rowsToSave);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	/* public function saveNext(DTO\Request\DataSet $request, DTO\Response $response) {
		try {
			$params = $request->getParameters();
			$rows = [];
			$this->dataSourceName = $this->getDataSourceName($params);

			foreach($params['editedRows'] as $row){
			    array_push($rows, $row);
			}
			$dataSet = new DataSet($this->dataSourceName, $rows);
	        $request = new DTO\Request\DataSet($dataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			
			$dataSet = new DataSet($this->dataSourceName, $rowsToSave);
	        $request = new DTO\Request\DataSet($dataSet, "", "", "");
			//$dataSetToSave = new DataSet($this->dataSourceName, $rowsToSave);
			$this->crudNext->persist($request, $response);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	} */
    
	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
			$params = $request->getParameters();
			
			if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('limit', $params) && !empty($params['limit'])) $limit = $params['limit'];
			else $limit = null; /*ajustado para funcionar propriedade showSelectAll*/
			
			if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('page', $params) && !empty($params['page'])) $page = $params['page'];
			else $page = 1;

			$this->dataSourceName = $params['dbname'];
			$filterCriteria = new FilterCriteria($this->dataSourceName, [], $page, $limit);
			$params = \Zeedhi\Framework\Util\Functions::arrayKeyExists('filter', $params) ? $params['filter'] : $params;

			/*Pega parâmetros caso esteja sendo passado em 'filter' ou 'conditions'*/ 
			if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('filter', $params)) $params = $params['filter'];
			else if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('conditions', $params)) $params = $params['conditions'];

			$this->buildFilterCriteria($filterCriteria, $params);

			$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
			$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);

			$this->setSearchConditions($newFilterCriteria, $params);
			$this->setSortConditions($newFilterCriteria, $params);
            $this->setDynamicFilter($newFilterCriteria, $params);
			$this->crudService->find($response, $newFilterCriteria, $params);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
	}

	private function buildFilterCriteria($filterCriteria, $params){
		foreach($params as $key => $val){
			if($key == strtoupper($key))
				if(is_array($val) && \Zeedhi\Framework\Util\Functions::arrayKeyExists('name', $val)) $filterCriteria->addCondition($val['name'], $val['operator'], \Zeedhi\Framework\Util\Functions::arrayKeyExists('value', $val) ? $val['value'] : null);
		}
	}

	private function setSearchConditions($filterCriteria, $params) {
		if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('search', $params)){
			$filterCriteria->addSearch($params['search']);
			$filterCriteria->setPageSize(null);
		}
		$this->buildSearchIn(Request::getQueryString(), $params, $filterCriteria);
	}
	
	private function setDynamicFilter($filterCriteria, $params) {
		if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('dynamic_filter', $params)) {
			$where = $this->crudService->createDynamicFilter($params['dynamic_filter'], "");
	        $filterCriteria->setWhereClause($where);
		}
	}

	private function setSortConditions($filterCriteria, $params) {
		preg_match_all("/(?:(?<=\border=))\w+[.]\w+/", Request::getQueryString(), $order);
		if(!empty($order)) {
			$order = reset($order);
			foreach($order as $value) {
				$val = explode('.', $value);
				$field = $val[0];
				$type = $val[1];
				$filterCriteria->addOrderBy($field, strtoupper($type));
			}			
		}
	}

	private function buildSearchIn(string $queryString, array &$parameters, FilterCriteria $filterCriteria) {
        if(preg_match_all("/(?:(?<=\bsearch_in=))\w+/", $queryString, $searchIn)) {
            $filterCriteria->setSearchIn(reset($searchIn));
            unset($parameters['search_in']);
        }
    }

	public function delete(DTO\Request $request, DTO\Response $response) {
		try {
			$params = $request->getParameters();
			$this->dataSourceName = $this->getDataSourceName($params);
			
			$dataSet = new DataSet($this->dataSourceName, $params['checkedRows']);
	        $request = new DTO\Request\DataSet($dataSet, "", "", "");
	        $rowsToDelete = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
	        
	        $dataSet = new DataSet($this->dataSourceName, $rowsToDelete);
            $this->crudService->delete($response, $dataSet);
		} catch (\Exception $e) {
			$errorMessage = $e->getMessage();
			foreach($this->errorCode as $key => $val){
				if(strpos($errorMessage, $key)) $errorMessage = $val;
			}
			$response->setError(new Error($errorMessage, 500));
		}
	}
	
	private function getDataSourceName($params){
	    $name = "";
	    if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('dataSourceName', $params) && $params['dataSourceName']) $name = str_replace("/","", $params['dataSourceName']);
	    return $name;   
	}
}