import {
  Metadata, Http, Loader,
} from '@zeedhi/core';
import {
  Form, Button, Grid, LoadingService,
} from '@zeedhi/common';

export class Pla01300Operador {
  private utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public prato = Loader.getInstance('Pla01300PratoController');
  
  public ajustarQtd = Loader.getInstance('Pla01300AjusteQuantidades');
  
  public importarFatorCorrecao = Loader.getInstance('Pla01300ImportarFatorConversao');
  
  public dadosReceita = Loader.getInstance('Pla01300DadosReceitaController');
  
  public receita = Loader.getInstance('ReceitaController');
  
  public revisarQtd = Loader.getInstance('Pla01300RevisarQuantidadesController');
  
  public revisarReceita = Loader.getInstance('Pla01309RevisaoAutomaticaController');
  
  public repReceita = Loader.getInstance('Pla01313ReplicacaoReceitasServicoController');
  
  public utilizacaoProdutos = Loader.getInstance('Pla01308UtilizacaoProdutosController');
  
  public qtdPessoa = Loader.getInstance('QuantidadePorPessoaController');
  
  public importacaoReceita = Loader.getInstance('ImportacaoReceitaController');
    
  public saveOperator() {
    const nome = this.prato.nameComponentOperador;
    const form = Metadata.getInstance<Form>('formGroupOper');
    const row = form.value;
    row.EMAIL = row.DSEMAILOPER;
    row.SENHA = row.CDSENHAOPERWEB;
     
    if (form.validate()) {
      LoadingService.show(' ');
      Http.post('/prato/autenticar', { requestType: 'Row', row }).then((permission) => {
        if (permission.data.dataset.permission.isValid !== false) {
          Http.post('/prato/saveOperator', { requestType: 'Row', row }).then((result) => {
            const msg = result.data.dataset.result.message;
            if (msg === 'INVALID_PASSWORD' || msg === 'operadorNaoCadastrado') {
              this.utilities.showAlert('Alert', 'pme::ERROR_SENHAUSUARIONEGADO', 'error');
              this.prato.operatorValid = false;
            } else if (msg === null) {
              this.prato.operatorValid = true;
              this.prato.closeModalGroupOperador();
              if (nome === 'ajustarQtd') this.ajustarQtd.openModal();
              else if (nome === 'fatorCor') this.importarFatorCorrecao.confirmaFatorCorrecao();
              else if (nome === 'delButtonPad') {
                const buttonDelete = Metadata.getInstance<Button>('delButtonPad');
                this.receita.deleteRecipeStandard({ component: buttonDelete });
              } else if (nome === 'addButtonPad') {
                const buttonAdd = Metadata.getInstance<Button>('addButtonPad');
                if (this.prato.alteraRecPadrao) {
                  this.receita.addRow({ component: buttonAdd });
                }
              } else if (nome === 'RECIPE_STANDARD') {
                const grid: Grid = Metadata.getInstance('RECIPE_STANDARD');
                this.gridEditController.onInlineEdit({ component: grid });
              } else if (nome === 'saveButtonRecipePadrao') {
                const buttonSaveDados: Button = Metadata.getInstance('saveButtonRecipePadrao');
                this.dadosReceita.salvaDadosRecipePlate({ component: buttonSaveDados });
              } else if (nome === 'salvar') {
                const revisarQtd: Button = Metadata.getInstance('salvar');
                this.revisarQtd.revisarQtd({ component: revisarQtd });
              } else if (nome === 'salvarRepReceita') {
                const salvarRepReceita: Button = Metadata.getInstance('salvarRepReceita');
                this.repReceita.confirmarAcao({ component: salvarRepReceita });
              } else if (nome === 'salvarUtilizacaoProd' || nome === 'addButtonUtil') {
                const alterar: Button = Metadata.getInstance(nome);
                this.utilizacaoProdutos.alterarReceitas({ component: alterar });
              } else if (nome === 'CALCULAR') {
                const calcular: Button = Metadata.getInstance('CALCULAR');
                this.qtdPessoa.procedureQtdPessoa({ component: calcular });
              } else if (nome === 'salvarRevisao') {
                const revisarReceita: Button = Metadata.getInstance('salvarRevisao');
                this.revisarReceita.confirmarAcoesRevisaoAutomatica({ component: revisarReceita });
              } else if (nome === 'importRecipePADRAO') {
                const importacaoText: Text = Metadata.getInstance('importRecipePADRAO');
                this.importacaoReceita.showModalImportPlate(importacaoText);
              }
            } else this.prato.operatorValid = false;
          }).catch((erro: string) => {
            this.utilities.showAlert('alert', `pme::${erro}`, 'error');
          });
        } else this.utilities.showAlert('alert', 'pme::unauthorizedOperator', 'error');
      }).catch((erro: string) => {
        this.utilities.showAlert('alert', `pme::${erro}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }
}
