import {
  IEventParam,
  Loader,
  Metadata,
  IDictionary,
  dayjs as moment,
  Datasource,
} from '@zeedhi/core';

import {
  GridEditable, Card, TextInput, Grid,
} from '@zeedhi/common';
  
export class Pla01400TipoPratoController {
  private tipoPratIsVisible = false;
  
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');
  
  public modalController = Loader.getInstance('ModalController');

  public utilities = Loader.getInstance('Utilities');
  
  public codNomeTipoPrato = '';

  public fillCode({ component }: IDictionary) {
    if (component.name === 'CDGRTIPOPR_editable_undefined') {
      if (component.value < 0) component.value = '0';
      component.value = component.value.padStart(3, '0');
    } else {
      if (component.value < 0) component.value = '0';
      component.value = component.value.padStart(2, '0');
    }
  }
  
  public toUpperCase({ component }: IEventParam<TextInput>) {
    component.value = component.value.toUpperCase();
  }
  
  public onChangeGrupTpPratoCurrentRow({ component }: IEventParam<Datasource>) {
    const cdGrupoTpPrat = component.currentRow.CDGRTIPOPR;
    
    if (cdGrupoTpPrat) {
      const gridTipoPrat = Metadata.getInstance<Grid>('tipoPratGridDetail');
      this.codNomeTipoPrato = `${component.currentRow.CDGRTIPOPR} - ${component.currentRow.NMGRTIPOPR}`;
      
      const conditions = [
        {
          name: 'CDGRTIPOPR',
          operator: '=',
          value: cdGrupoTpPrat,
        },
      ];
      
      gridTipoPrat.datasource.filter = conditions;
      // gridTipoPrat.datasource.get();
      this.cardTipoPratVisible();
    }
  }
  
  private cardTipoPratVisible() {
    const card = Metadata.getInstance<Card>('tipoPratoCard');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.tipoPratIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.tipoPratIsVisible = false;
    }
  }
  
  public closeCardTipoPrato() {
    const card = Metadata.getInstance<Card>('tipoPratoCard');
    
    card.isVisible = false;
    this.tipoPratIsVisible = false;
  }
  
  get gridColsReturnValue(): number {
    return this.tipoPratIsVisible ? 6 : 12;
  }
  
  public addTipoPrato() {
    const grid: GridEditable = Metadata.getInstance('gridTpPratCard');
    this.gridEditController.addRow({ component: grid });
  }

  public saveGrupoTipoPrat() {
    const gridGrupoTipoPrat: GridEditable = Metadata.getInstance('gridGrupoTipoPrat');
    const editedRows: IDictionary = gridGrupoTipoPrat.getEditedRows(false, true);

    if (gridGrupoTipoPrat.isGridValid(true) && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDGRTIPOPR = row.CDGRTIPOPR.padStart(2, '0');
        }
      });

      this.crudController.saveRows(editedRows, 'grtipopr', gridGrupoTipoPrat).then(() => {
        this.gridEditController.defaultState(gridGrupoTipoPrat);
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public deleteCheckedRowsGpTpPrat() {
    const gridGrupoTipoPrat: GridEditable = Metadata.getInstance('gridGrupoTipoPrat');
    this.gridEditController.deleteCheckedRows({ component: gridGrupoTipoPrat });
  }

  public fillCodeTipoPrato({ component }: IEventParam<TextInput>) {
    component.value = component.value.padStart(2, '0');
  }
  
  public saveTipoPratCard() {
    const gridTpPratCard: GridEditable = Metadata.getInstance('gridTpPratCard');
    const editedRows: IDictionary = gridTpPratCard.getEditedRows(false, true);

    if (gridTpPratCard.isGridValid(true) && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDTIPOPRAT = row.CDTIPOPRAT.padStart(2, '0');
          row.NMREDUZIDO = row.NMREDUZIDO ? row.NMREDUZIDO.toUpperCase() : '';
          row.CDGRTIPOPR = row.CDGRTIPOPR ? row.CDGRTIPOPR : '';
          row.IDSOBRLIMPTP = row.IDSOBRLIMPTP ? row.IDSOBRLIMPTP : 'N';
          row.IDMEDICAOKG = row.IDMEDICAOKG ? row.IDMEDICAOKG : 'N';
          row.DTULTATUTPRA = moment().format('DD/MM/YYYY');
        } else row.NMREDUZIDO = row.NMREDUZIDO ? row.NMREDUZIDO.toUpperCase() : '';
      
        this.crudController.saveRows(editedRows, 'tipopratPla', gridTpPratCard).then(() => {
          this.gridEditController.defaultState(gridTpPratCard);
          this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
          // const tipoPratGridDetail: Grid = Metadata.getInstance('tipoPratGridDetail');
          // tipoPratGridDetail.datasource.get();
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      });
    }
  }
}
