import {
  Metadata, Loader, IEventParam, IDictionary, Http, Config, I18n, Datasource, Mask,
} from '@zeedhi/core';
import {
  Select, GridEditable, Form, Button, Card, DialogService, LoadingService, Number as NumberInput,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme04000RealocacaoComensaisEspeciaisPrato {
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  public crudController = Loader.getInstance('CrudController');
  
  private comensaisIsVisible = false;
  
  public selectedPrato = '';
  
  public filterTipoCardapio({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value ? unidade.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public changeUnidade() {
    const tipoCard = Metadata.getInstance<Select>('TPCARD');
    tipoCard.setValue('');
  }
  
  public filterAgregada({ component }: IEventParam<Select>) {
    const grid = Metadata.getInstance<GridEditable>('COMESPPRAT');
    component.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    component.datasource.get();
    grid.datasource.updateData([]);
    grid.datasource.total = 0;
  }
  
  public filterComensaisEspeciais() {
    const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA');
    const grid: GridEditable = Metadata.getInstance('REALCOMESPPRAT');
    const tipoCard = Metadata.getInstance<Select>('TPCARD').selectValue;
    const form = Metadata.getInstance<Form>('formFilter');
    if (form.validate()) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade.value ? unidade.value : '',
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: tipoCard ? tipoCard.CDTIPOCARD : '',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else {
      grid.datasource.data = [];
      grid.datasource.selectedRows = [];
    }
  }
  
  public addRow({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formFilter');
    if (form.validate() && !grid.editing) {
      this.gridEditableController.addRow({ component: grid });
      if (grid.name === 'COMESPPRAT') {
        setTimeout(() => {
          const qt = Metadata.getInstance<NumberInput>('QTCOMESPRAT_editable_');
          qt.value = null;
        }, 100);
      }
    }
  }
  
  public cancelEditingGrid({ component }: IEventParam<Button>) {
    this.gridEditableController.cancelEditing({ component });
  }
  
  public cancelEditingComensais({ component }: IEventParam<Button>) {
    this.gridEditableController.cancelEditing({ component });
    // const qtComensais = Metadata.getInstance<NumberInput>('QTCOMESPRAT');
    // qtComensais.value = null;
  }
  
  public saveComEspPrat({ component }: IEventParam<Button>) {
    const grid = Metadata.getInstance<GridEditable>('COMESPPRAT');
    const gridPrincipal = Metadata.getInstance<GridEditable>('REALCOMESPPRAT');
    const editedRows: IDictionary = grid.getEditedRows(false, true);

    if (grid.isGridValid(true) && editedRows.length > 0) {
      if (grid) grid.datasource.loading = true;
      const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA');
      const tpcard = Metadata.getInstance<Select>('TPCARD');
      editedRows.forEach((row: IDictionary) => {
        row.CDREALCOMESPPRATPME = gridPrincipal.datasource.currentRow.CDREALCOMESPPRATPME;
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.CDFILIAL = unidade.value;
        row.CDTIPOCARD = tpcard.value;
        if (!row.CDTIPOPRAT && !row.CDPRATO && !row.CDFILAUXPRAT) {
          const l = grid.datasource.data.length;
          row.CDTIPOPRAT = grid.datasource.data[l - 1].CDTIPOPRAT;
          row.CDPRATO = grid.datasource.data[l - 1].CDPRATO;
          row.CDFILAUXPRAT = grid.datasource.data[l - 1].CDFILAUXPRAT;
          row.CDCOMESPPRATPME = grid.datasource.data[l - 1].CDCOMESPPRATPME;
          row.__is_new = false;
        }
      });
      LoadingService.show(' ');
      Http.post('/saveComEspPrat', { editedRows }).then((response: IDictionary) => {
        if (response.data.length === 0) {
          // grid.datasource.get();
          this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
          this.gridEditableController.cancelEditing({ component });
          // this.gridEditableController.defaultState(grid);
        } else {
          const mensagem = response.data.dataset.message.message;
          if (mensagem.indexOf('Impossível adicionar. Registro já existe.') === 0) {
            this.utilities.showAlert('alertError', I18n.translate('Impossível adicionar. Registro já existe.'), 'error');
          } else if (mensagem.indexOf('Não é possível realocar comensais para o próprio prato.') === 0) {
            this.utilities.showAlert(
              'alertError', I18n.translate('Não é possível realocar comensais para o próprio prato.'), 'error',
            );
          }
        }
      }).catch((error: IDictionary) => {
        this.utilities.showAlert('alertError', error, 'error');
      }).finally(() => {
        LoadingService.hide();
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  public saveRealComEspPrat({ component }: IEventParam<Button>) {
    const grid = Metadata.getInstance<GridEditable>('REALCOMESPPRAT');
    const editedRows: IDictionary = grid.getEditedRows(false, true);
    const form = Metadata.getInstance<Form>('formFilter');
    if (grid.isGridValid(true) && editedRows.length > 0 && form.validate()) {
      const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA');
      const tpcard = Metadata.getInstance<Select>('TPCARD');
      if (unidade.value === null || unidade.value === undefined || tpcard.value === null || tpcard.value === undefined) {
        this.utilities.showAlert('alertError',
          I18n.translate('Informa a unidade e o tipo de cardápio para adicionar.'), 'error');
      } else {
        if (grid) grid.datasource.loading = true;
        editedRows.forEach((row: IDictionary) => {
          row.CDFILCENTPME = this.appController.getCdCentral();
          row.CDFILIAL = unidade.value;
          row.CDTIPOCARD = tpcard.value;
        });
        Http.post('/saveRealComEspPrat', { editedRows }).then((response: IDictionary) => {
          if (response.data.length === 0) {
            // grid.datasource.get();
            this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
            this.gridEditableController.cancelEditing({ component });
          } else {
            const mensagem = response.data.dataset.message.message;
            if (mensagem.indexOf('Impossível adicionar. Registro já existe.') === 0) {
              this.utilities.showAlert('alertError', I18n.translate('Impossível adicionar. Registro já existe.'), 'error');
            }
          }
        }).catch((error: IDictionary) => {
          this.utilities.showAlert('alertError', error, 'error');
        }).finally(() => {
          if (grid) grid.datasource.loading = false;
        });
      }
    }
  }
  
  public clickRealComEspPrat({ component }: IEventParam<Datasource>) {
    const pratoSelecionado = !!component.currentRow.CDPRATO;
    const card = Metadata.getInstance<Card>('comensaisCard');

    if (pratoSelecionado) {
      const grid: GridEditable = Metadata.getInstance('COMESPPRAT');
      grid.selectedRows = [];
      // grid.columns[0].loading = true;
      const cdprato = Mask.getValueWithMask(component.currentRow.CDPRATO, this.utilities.maskPlate());
      this.selectedPrato = `${component.currentRow.CDTIPOPRAT} - ${component.currentRow.NMTIPOPRAT} `
        + `| ${cdprato} - ${component.currentRow.NMPRATO}`;
    
      const conditions = [
        {
          name: 'CDREALCOMESPPRATPME',
          operator: '=',
          value: component.currentRow.CDREALCOMESPPRATPME,
        },
      ];
      grid.datasource.filter = conditions;
      // grid.datasource.get().then(() => {
      this.cardComensaisVisible();
      // });
    } else {
      const cdprato = Mask.getValueWithMask(component.currentRow.CDPRATO, this.utilities.maskPlate());
      this.selectedPrato = `${component.currentRow.CDTIPOPRAT} - ${component.currentRow.NMTIPOPRAT} `
        + `| ${cdprato} - ${component.currentRow.NMPRATO}`;
      card.isVisible = false;
      this.comensaisIsVisible = false;
    }
  }
  
  private cardComensaisVisible() {
    const card = Metadata.getInstance<Card>('comensaisCard');
    try {
      if (!this.gridEditableController.newRow) {
        card.isVisible = true;
        this.comensaisIsVisible = true;
      } else {
        const btn = Metadata.getInstance<Button>('excluiPrato');
        this.cancelEditingComensais({ component: btn });
        // .then(() => {
        card.isVisible = true;
        this.comensaisIsVisible = true;
        // });
      }
    } catch (Error) {
      card.isVisible = false;
      this.comensaisIsVisible = false;
    }
  }
  
  get gridColsRealoc(): number {
    return this.comensaisIsVisible ? 5 : 12;
  }
  
  get gridColsComensais(): number {
    return this.comensaisIsVisible ? 7 : 12;
  }
  
  public closeCardComensais() {
    const card = Metadata.getInstance<Card>('comensaisCard');
    const gridComensais: GridEditable = Metadata.getInstance('COMESPPRAT');
    this.gridEditableController.cancelEditing({ component: gridComensais });
    
    card.isVisible = false;
    this.comensaisIsVisible = false;
  }
  
  public async deleteComEspPrat({ component }: IDictionary) {
    const grid: GridEditable = Metadata.getInstance('COMESPPRAT');
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      await DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                // const buttonDel: GridEditable = Metadata.getInstance('yesDelete');
                LoadingService.show(' ');
                this.crudController.deleteRows(checkedRows, 'comEspPrat').then(() => {
                  grid.cancelEditedRows();
                  // grid.datasource.get();
                  grid.selectedRows = [];
                  this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
                }).catch((error: string) => {
                  this.utilities.showAlert('alert', `pme::${error}`, 'error');
                }).finally(() => {
                  LoadingService.hide();
                  DialogService.hide();
                });
              },
            },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteRealComEspPrat() {
    LoadingService.show(' ');
    const grid = Metadata.getInstance<GridEditable>('REALCOMESPPRAT');
    const checkedRows = grid.selectedRows;
    Http.post('/deleteRealComEspPrat', { requestType: 'Row', checkedRows }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        grid.datasource.get();
        grid.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'Excluído com sucesso!', 'success');
        DialogService.hide();
      } else {
        const mensagem = response.data.dataset.message[0];
        if (mensagem.indexOf('ERROR_TEM_PRATO_REGISTRO') === 0) {
          this.utilities.showAlert('alertError',
            I18n.translate('Operação bloqueada! Registro possui prato relacionado.'), 'error');
          DialogService.hide();
        }
      }
    }).catch((error) => {
      grid.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    }).finally(() => {
      this.closeCardComensais();
      LoadingService.hide();
    });
  }
  
  public setFilterPrato({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA');
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    component.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value,
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: nivel,
      },
    ];
    component.datasource.get();
  }
  
  public confirmDeleteRealComEspPrat({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      DialogService.show({
        name: 'excluiGrupo',
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.deleteRealComEspPrat() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
}
