import {
  Loader, Metadata, IDictionary, IEventParam, Http, I18n,
} from '@zeedhi/core';

import {
  Form, Select, Grid, IDate, IGridEvent, GridEditable, TextInput, Button,
  DialogService, Card, ModalService, Modal, LoadingService,
} from '@zeedhi/common';

export class Pme12200FreqProdsPeriodo {
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalGrupoPrato!: Modal;
  
  public modalVigencia!: Modal;
  
  public selectedGrupo = '';
  
  private selectedFreq = '';
  
  private produtoIsVisible = false;
  
  private descricao = '';
  
  private selectedAgrup = '';
  
  private newValidVigencia = true;
  
  public setFilterTpCardapio({ component }: IDictionary) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('SERVICOPROD');
    const descricao: IDictionary = Metadata.getInstance('descriptionProd');
    const form = Metadata.getInstance<Form>('formFreqProdPer');
    
    if (component.name === 'TIPOCARDPROD') {
      servico.setValue('');
      this.selectedFreq = '';
      form.value.describeCard = null;
      form.value.finalDateProd = null;
      form.value.initialDateProd = null;
      descricao.value = '';
      this.descricao = '';
      this.setFilterGrids();
    } else if (component.name === 'SERVICOPROD') {
      this.selectedFreq = '';
      form.value.describeCard = null;
      form.value.finalDateProd = null;
      form.value.initialDateProd = null;
      descricao.value = '';
      this.descricao = '';
      this.setFilterGrids();
    }
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
    let valueCard;
    
    if (cdtipoCard.returnObject === true && cdtipoCard.value !== null) valueCard = cdtipoCard.value.CDTIPOCARD;
    else valueCard = cdtipoCard.value;
    
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? valueCard : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
    fim.value = null;
    this.selectedFreq = '';
  }
  
  public changeVigencia() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARDPROD');
    const servico: IDictionary = Metadata.getInstance('SERVICOPROD');
    const ini: IDate = Metadata.getInstance<IDate>('initialDateProd');
    const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
    const form = Metadata.getInstance<Form>('formFreqProdPer');
    if (form.validate()) {
      if (ini.value !== null || (ini.value !== null && fim.value !== null && this.utilities.dateIsValid(ini, fim))) {
        if (tipoCard.value && servico.value) {
          const filter: IDictionary = [];
          this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', tipoCard.value);
          this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
          this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', servico.value);
          
          Http.post('/find/vigenciaFreqProd', { requestType: 'FilterData', filter }).then((response) => {
            const result = response.data.data;
            let nrFreqProd = '';
            let desc = '';
            let finalDate = fim.value;
            result.forEach((vigencia: IDictionary) => {
              // && vigencia.DTVIGENCIAFIM === fim.value
              if (vigencia.DTVIGENCIA === ini.value && vigencia.DTVIGENCIAFIM === fim.value) {
                nrFreqProd = vigencia.NRFREQPRODMESPAE;
                desc = vigencia.DESCRICAO;
                finalDate = vigencia.DTVIGENCIAFIM;
              }
            });
            this.selectedFreq = nrFreqProd;
            this.descricao = desc;
            Metadata.getInstance<IDictionary>('descriptionProd').value = desc;
            fim.value = finalDate;
            if (this.selectedFreq === '' && !this.utilities.validateVigenciaDates(result, ini, fim)) {
              this.utilities.showAlert('alertError',
                'A(s) data(s) selecionada(s) para a vigência intercalam com a data de outra vigência.', 'error');
              this.newValidVigencia = false;
            } else {
              this.newValidVigencia = true;
            }
          }).finally(() => {
            this.setFilterGrids();
          });
        } else {
          this.utilities.showAlert('alertError', 'Tipo de Cardápio e Serviço não selecionado', 'warning');
          this.setFilterGrids();
        }
      } else this.setFilterGrids();
    }
  }
  
  private async createModalVigencia() {
    const modalPath = 'pme/restricoesContratuais/cadastros/modals/gridVigenciasProd';
    this.modalVigencia = await ModalService.createFromJsonPost('', { jsonPath: modalPath });
  }
  
  public openVigenciaModal() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARDPROD');
    const servico: IDictionary = Metadata.getInstance('SERVICOPROD');
    if (tipoCard.value && servico.value) {
      if (!this.modalVigencia) {
        this.createModalVigencia().then(() => {
          this.modalVigencia.show();
          this.setFilterVigencia();
        });
      } else {
        this.modalVigencia.show();
        this.setFilterVigencia();
      }
    } else {
      this.utilities.showAlert('alertError', 'Tipo de Cardápio e Serviço não selecionado', 'warning');
    }
  }
  
  public setFilterVigencia() {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
    const cdservfili = Metadata.getInstance<Select>('SERVICOPROD');
    
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
    ];
      
    if (document.getElementsByName('gridVigenciaFreqProd').length > 0) {
      const gridVigenciaProd = Metadata.getInstance<Grid>('gridVigenciaFreqProd');
      if (gridVigenciaProd) {
        gridVigenciaProd.datasource.filter = filter;
        gridVigenciaProd.datasource.get();
      }
    }
  }
  
  public setVigencia({ row }: IGridEvent) {
    if (row) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDateProd');
      const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
      ini.value = row.DTVIGENCIA;
      fim.value = row.DTVIGENCIAFIM;
      if (row.NRFREQPRODMESPAE) {
        this.selectedFreq = row.NRFREQPRODMESPAE;
      }
      const descricao: IDictionary = Metadata.getInstance('descriptionProd');
      descricao.value = row.DESCRICAO;
      this.descricao = row.DESCRICAO;
      this.changeVigencia();
      this.modalVigencia.hide();
    }
  }
  
  public changeDescription({ component }: IEventParam<TextInput>) {
    if (this.selectedFreq !== '' && component.value !== this.descricao) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDateProd');
      const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
      const descricao: IDictionary = Metadata.getInstance('descriptionProd');
      const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
      const cdservfili = Metadata.getInstance<Select>('SERVICOPROD');
      if (this.selectedFreq !== '') {
        const vigencia = {
          NRFREQPRODMESPAE: this.selectedFreq,
          DTVIGENCIA: ini.value,
          DTVIGENCIAFIM: fim.value,
          DESCRICAO: descricao.value,
          CDFILCENTPME: this.appController.getCdCentral(),
          CDTIPOCARD: cdtipoCard.value,
          CDSERVFILI: cdservfili.value,
          __is_new: false,
        };
        
        this.crudController.saveRows([vigencia], 'vigenciaFreqProd').then(() => {
          this.descricao = descricao.value;
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public setFilterGrids() {
    if (document.getElementsByName('gridFreqPorProd').length > 0) {
      const gridFreqPorProd: IDictionary = Metadata.getInstance('gridFreqPorProd');
      
      if (this.selectedFreq === '') {
        gridFreqPorProd.datasource.filter = [];
        gridFreqPorProd.datasource.data = [];
      } else {
        const filter = [
          {
            name: 'NRFREQPRODMESPAE',
            operator: '=',
            value: this.selectedFreq,
          },
        ];
        gridFreqPorProd.datasource.filter = filter;
        gridFreqPorProd.datasource.get();
      }
    }
    if (document.getElementsByName('gridFreqProdPorGrupoPratos').length > 0) {
      const gridFreqPorGrupoPratos: IDictionary = Metadata.getInstance('gridFreqProdPorGrupoPratos');
      if (this.selectedFreq === '') {
        gridFreqPorGrupoPratos.datasource.filter = [];
        gridFreqPorGrupoPratos.datasource.data = [];
      } else {
        const filterGrupo = [
          {
            name: 'NRFREQPRODMESPAE',
            operator: '=',
            value: this.selectedFreq,
          },
        ];
        
        gridFreqPorGrupoPratos.datasource.filter = filterGrupo;
        gridFreqPorGrupoPratos.datasource.get();
      }
    }
  }
  
  public savePorProd() {
    const gridFreqPorProd: GridEditable = Metadata.getInstance('gridFreqPorProd');
    const editedRows: IDictionary = gridFreqPorProd.getEditedRows(false, true);

    if (this.selectedFreq === '') {
      if (!this.newValidVigencia) {
        this.utilities.showAlert('alertError',
          'A(s) data(s) selecionada(s) para a vigência intercalam com a data de outra vigência.', 'error');
      } else if (gridFreqPorProd.isGridValid(true) && editedRows.length > 0) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDateProd');
        const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
        const descricao: IDictionary = Metadata.getInstance('descriptionProd');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
        const cdservfili = Metadata.getInstance<Select>('SERVICOPROD');
        
        Http.post('/returnCodFreqProd', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRFREQPRODMESPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          
          this.crudController.saveRows([vigencia], 'vigenciaFreqProd', gridFreqPorProd).then((result: IDictionary) => {
            if (result.data.dataset) {
              this.selectedFreq = vigencia.NRFREQPRODMESPAE;
              this.saveItFreqProd(gridFreqPorProd);
            } else {
              const error = response.data.error;
              if (error.indexOf('UN_FREQPRODMESPAE') > 0) {
                this.utilities.showAlert('alertError',
                  'Vigência já existe. Informe outra data inicial de vigência.', 'error');
              } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      this.saveItFreqProd(gridFreqPorProd);
    }
  }
  
  private saveItFreqProd(grid: GridEditable) {
    const editedRows: IDictionary = grid.getEditedRows(false, true);

    if (grid.isGridValid(true) && editedRows.length > 0) {
      let valid = true;
      editedRows.forEach((row: IDictionary) => {
        row.NRFREQPRODMESPAE = this.selectedFreq;
        if (row.NRMINFREQPRODMES > row.NRMAXFREQPRODMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::minMesMaiorMaxMes', 'error');
        }
        if (row.NRMAXFREQPRODSEM > row.NRMAXFREQPRODMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxMesMenorMaxSem', 'error');
        }
        if (row.NRMINFREQPRODMES <= 0 || row.NRMAXFREQPRODMES <= 0 || row.NRMAXFREQPRODSEM <= 0) {
          valid = false;
          this.utilities.showAlert('alert', 'Quantidade deve ser maior que zero.', 'error');
        }
        if (row.NRMAXFREQPRODSEM > row.NRMINFREQPRODMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxSemMaiorMinMes', 'error');
        }
      });
      
      if (valid) {
        Http.post('/saveItFreqProd', { editedRows }).then((result) => {
          if (result.data.length === 0) {
            this.setFilterGrids();
            this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
            this.gridEditController.cancelEditing({ component: grid });
          } else {
            const mensagem = result.data.dataset.mensagem[0];
            if (mensagem.indexOf('ERROR_PROD_CADASTRADO_VIG') === 0) {
              this.utilities.showAlert('alertError',
                I18n.translate('Produto já cadastrado para a vigência selecionada.'), 'error');
            } else {
              this.utilities.showAlert('alert', `pme::${result.data}`, 'error');
            }
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public savePorGrupoPrato() {
    const gridFreqPorGrupoPratos: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
    if (this.selectedFreq === '') {
      if (!this.newValidVigencia) {
        this.utilities.showAlert('alertError',
          'A(s) data(s) selecionada(s) para a vigência intercalam com a data de outra vigência.', 'error');
      } else if (gridFreqPorGrupoPratos.isGridValid(true)) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDateProd');
        const fim: IDate = Metadata.getInstance<IDate>('finalDateProd');
        const descricao: IDictionary = Metadata.getInstance('descriptionProd');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
        const cdservfili = Metadata.getInstance<Select>('SERVICOPROD');
        
        Http.post('/returnCodFreqProd', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRFREQPRODMESPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          const dtNm = 'vigenciaFreqProd';
          this.crudController.saveRows([vigencia], dtNm, gridFreqPorGrupoPratos).then((result: IDictionary) => {
            if (result.data.dataset) {
              this.selectedFreq = vigencia.NRFREQPRODMESPAE;
              this.saveItFreqGrPrat(gridFreqPorGrupoPratos);
            } else {
              const error = response.data.error;
              if (error.indexOf('UN_FREQPRODMESPAE') > 0) {
                this.utilities.showAlert('alertError',
                  'Vigência já existe. Informe outra data inicial de vigência.', 'error');
              } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      const gridProd: GridEditable = Metadata.getInstance('PRODUTO');
      this.saveItFreqGrPrat(gridProd);
    }
  }
  
  private saveItFreqGrPrat(grid: GridEditable) {
    const editedRows: IDictionary = grid.getEditedRows();
    if (editedRows.length === 0) {
      const form = Metadata.getInstance<Form>('formFreqProdGrupoPrato');
      if (form.validate()) {
        const row = form.value;
        let valid = true;
        row.__is_new = true;
        row.NRFREQPRODMESPAE = this.selectedFreq;
        row.NRGRUPOPRATOPAE = row.NMGRUPOPRATOPAE.NRGRUPOPRATOPAE;
        row.CDPRODUTO = row.NMPRODUTO.CDPRODUTO;
        if (row.NRMINFREQGRPRATMES > row.NRMAXFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::minMesMaiorMaxMes', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM > row.NRMAXFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxMesMenorMaxSem', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM <= 0 || row.NRMAXFREQGRPRATMES <= 0 || row.NRMINFREQGRPRATMES <= 0) {
          valid = false;
          this.utilities.showAlert('alert', 'Quantidade deve ser maior que zero.', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM > row.NRMINFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxSemMaiorMinMes', 'error');
        }
        if (valid) {
          Http.post('/saveItFreqProdGrPr', { editedRows: [row] }).then((result) => {
            if (result.data.length === 0) {
              this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
              const gridFreqPorGrupoPratos: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
              const filter = [
                {
                  name: 'NRFREQPRODMESPAE',
                  operator: '=',
                  value: this.selectedFreq,
                },
              ];
              gridFreqPorGrupoPratos.datasource.filter = filter;
              const nrAgrup = row.NMAGRUPGRPRAT.NRAGRUPGRPRAT;
              const nrGrupo = row.NRGRUPOPRATOPAE;
              gridFreqPorGrupoPratos.datasource.get().then(() => {
                const grupoAdicionado = gridFreqPorGrupoPratos.datasource.data.find(
                  (d: IDictionary) => d.NRAGRUPGRPRAT === nrAgrup && d.NRGRUPOPRATOPAE === nrGrupo,
                );
                if (grupoAdicionado) gridFreqPorGrupoPratos.datasource.currentRow = grupoAdicionado;
                this.carregaProduto();
              });
              this.closeGrupoPratoModalAdd();
            } else {
              const mensagem = result.data.dataset.mensagem[0];
              if (mensagem.indexOf('ERROR_PROD_GRUP_CADASTRADO_VIG') === 0) {
                this.utilities.showAlert('alertError',
                  I18n.translate('Produto já cadastrado para o grupo na vigência selecionada.'), 'error');
              } else {
                this.utilities.showAlert('alert', `pme::${result.data}`, 'error');
              }
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        }
      }
    } else if (editedRows.length > 0) {
      const gridFreqPorGrupoPratos: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
      const nrgrupo = gridFreqPorGrupoPratos.datasource.currentRow.NRGRUPOPRATOPAE;

      if (gridFreqPorGrupoPratos.isGridValid(true)) {
        let valid = true;
        editedRows.forEach((row: IDictionary) => {
          row.NRFREQPRODMESPAE = this.selectedFreq;
          row.NRGRUPOPRATOPAE = nrgrupo;
          row.CDPRODUTO = row.CDPRODUTO.CDPRODUTO ? row.CDPRODUTO.CDPRODUTO : row.CDPRODUTO;
          if (row.NRMINFREQGRPRATMES > row.NRMAXFREQGRPRATMES) {
            valid = false;
            this.utilities.showAlert('alert', 'pme::minMesMaiorMaxMes', 'error');
          }
          if (row.NRMAXFREQGRPRATSEM > row.NRMAXFREQGRPRATMES) {
            valid = false;
            this.utilities.showAlert('alert', 'pme::maxMesMenorMaxSem', 'error');
          }
          if (row.NRMAXFREQGRPRATSEM <= 0 || row.NRMAXFREQGRPRATMES <= 0 || row.NRMINFREQGRPRATMES <= 0) {
            valid = false;
            this.utilities.showAlert('alert', 'Quantidade deve ser maior que zero.', 'error');
          }
          if (row.NRMAXFREQGRPRATSEM > row.NRMINFREQGRPRATMES) {
            valid = false;
            this.utilities.showAlert('alert', 'pme::maxSemMaiorMinMes', 'error');
          }
        });
        if (valid) {
          Http.post('/saveItFreqProdGrPr', { editedRows }).then((result) => {
            if (result.data.length === 0) {
              this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'success');
              this.gridEditController.cancelEditing({ component: grid });
            } else {
              const mensagem = result.data.dataset.mensagem[0];
              if (mensagem.indexOf('ERROR_PROD_GRUP_CADASTRADO_VIG') === 0) {
                this.utilities.showAlert('alertError',
                  I18n.translate('Produto já cadastrado para o grupo na vigência selecionada.'),
                  'error');
              } else {
                this.utilities.showAlert('alert', `pme::${result.data}`, 'error');
              }
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        }
      }
    }
  }
  
  public setFilterAgrup({ component, event }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARDPROD');
    const cdservfili = Metadata.getInstance<Select>('SERVICOPROD');
    const nmgrupo = Metadata.getInstance<Select>('NMGRUPOPRATOPAE');
    if (event && event.type === 'blur') this.selectedAgrup = component.value.NRAGRUPGRPRAT;
    
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
      {
        name: 'NRAGRUPGRPRAT',
        operator: '=',
        value: this.selectedAgrup ? this.selectedAgrup : this.selectedAgrup,
      },
    ];
    
    nmgrupo.datasource.filter = filter;
    nmgrupo.datasource.search = '';
    nmgrupo.datasource.searchIn = [];
    nmgrupo.datasource.get().then((response) => {
      if (event && event.type === 'focus') component.datasource.data = response;
    });
  }
  
  public closeCardProduto() {
    const card = Metadata.getInstance<Card>('prodCard');
    const gridProd: GridEditable = Metadata.getInstance('PRODUTO');
    this.gridEditController.cancelEditing({ component: gridProd });
    
    card.isVisible = false;
    this.produtoIsVisible = false;
  }
  
  public async deleteProduto({ component }: IDictionary) {
    const grid: GridEditable = Metadata.getInstance('PRODUTO');
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      await DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                // const buttonDel: GridEditable = Metadata.getInstance('yesDelete');
                LoadingService.show(' ');
                this.crudController.deleteRows(checkedRows, 'itFrequenciaProdGrPr').then(() => {
                  if (checkedRows.length === grid.datasource.data.length) {
                    this.reloadProduto();
                    this.setFilterGrids();
                    this.closeCardProduto();
                  } else {
                    this.reloadProduto();
                  }
                  grid.selectedRows = [];
                  this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
                }).catch((error: string) => {
                  this.utilities.showAlert('alert', `pme::${error}`, 'error');
                }).finally(() => {
                  LoadingService.hide();
                  DialogService.hide();
                });
              },
            },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public async deleteFreqProdGrPr({ component }: IDictionary) {
    const grid = Metadata.getInstance<GridEditable>('gridFreqProdPorGrupoPratos');
    const checkedRows = grid.selectedRows;
    checkedRows.forEach((row: IDictionary) => {
      row.NRFREQPRODMESPAE = this.selectedFreq;
    });
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else {
      await DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                // const buttonDel: GridEditable = Metadata.getInstance('yesDelete');
                LoadingService.show(' ');
                Http.post('/excluiFreqProdGrPr', { requestType: 'Row', checkedRows }).then((response: IDictionary) => {
                  if (response.data.length === 0) {
                    grid.datasource.get();
                    grid.selectedRows = [];
                    this.utilities.showAlert('alertSuccess', 'Excluído com sucesso!', 'success');
                    DialogService.hide();
                  } else {
                    const mensagem = response.data.dataset.mensagem[0];
                    this.utilities.showAlert('alertError', `pme::${mensagem}`, 'error');
                    DialogService.hide();
                  }
                }).catch((error) => {
                  grid.selectedRows = [];
                  this.utilities.showAlert('alertError', error, 'error');
                  DialogService.hide();
                }).finally(() => {
                  LoadingService.hide();
                  this.closeCardProduto();
                });
              },
            },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public async reloadProduto() {
    const produto: GridEditable = Metadata.getInstance('PRODUTO');
    const grupo: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
    
    const conditions = [
      {
        name: 'NRGRUPOPRATOPAE',
        operator: '=',
        value: grupo.datasource.currentRow.NRGRUPOPRATOPAE,
      },
      {
        name: 'NRFREQPRODMESPAE',
        operator: '=',
        value: this.selectedFreq,
      },
    ];
    produto.datasource.filter = conditions;
    produto.datasource.get();
  }
  
  public async cancelEditingTelaProduto({ component }: Button) {
    this.gridEditController.cancelEditing({ component });
    const produto: GridEditable = Metadata.getInstance('PRODUTO');
    const grupo: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
    
    const conditions = [
      {
        name: 'NRGRUPOPRATOPAE',
        operator: '=',
        value: grupo.datasource.currentRow.NRGRUPOPRATOPAE,
      },
      {
        name: 'NRFREQPRODMESPAE',
        operator: '=',
        value: this.selectedFreq,
      },
    ];
    produto.datasource.filter = conditions;
    produto.datasource.get();
  }
  
  private cardProdVisible() {
    const card = Metadata.getInstance<Card>('prodCard');
    try {
      card.isVisible = true;
      this.produtoIsVisible = true;
    } catch (Error) {
      card.isVisible = false;
      this.produtoIsVisible = false;
    }
  }
  
  get gridColsReturnValue(): number {
    return this.produtoIsVisible ? 5 : 12;
  }
  
  public async carregaProduto() {
    const grupo: GridEditable = Metadata.getInstance('gridFreqProdPorGrupoPratos');
    const grupoSelecionado = !!grupo.datasource.currentRow.NRGRUPOPRATOPAE;
    const card = Metadata.getInstance<Card>('prodCard');

    if (grupoSelecionado) {
      const prod: GridEditable = Metadata.getInstance('PRODUTO');
      this.selectedGrupo = `${grupo.datasource.currentRow.CDGRUPOPRATOPAE} - ${grupo.datasource.currentRow.NMGRUPOPRATOPAE}`;
    
      const conditions = [
        {
          name: 'NRGRUPOPRATOPAE',
          operator: '=',
          value: grupo.datasource.currentRow.NRGRUPOPRATOPAE,
        },
        {
          name: 'NRFREQPRODMESPAE',
          operator: '=',
          value: this.selectedFreq,
        },
      ];
      prod.datasource.filter = conditions;
      prod.datasource.loading = true;
      prod.datasource.get();
      this.cardProdVisible();
    } else {
      this.selectedGrupo = `${grupo.datasource.currentRow.CDGRUPOPRATOPAE} - ${grupo.datasource.currentRow.NMGRUPOPRATOPAE}`;
      card.isVisible = false;
      this.produtoIsVisible = false;
    }
  }
  
  private async createModalAddGrupoPrato() {
    const modalPath = 'pme/restricoesContratuais/cadastros/modals/freq_prods_add';
    this.modalGrupoPrato = await ModalService.createFromJsonPost('', { jsonPath: modalPath });
  }
  
  public addGrupoPrato() {
    const form = Metadata.getInstance<Form>('formFreqProdPer');
    if (this.utilities.centralIsValid() && form.validate()) {
      if (!this.modalGrupoPrato) {
        this.createModalAddGrupoPrato().then(() => {
          this.modalGrupoPrato.show();
        });
      } else {
        this.modalGrupoPrato.show();
      }
    }
  }
  
  public closeModalVigencia() {
    this.modalVigencia.hide();
  }
  
  public closeGrupoPratoModalAdd() {
    const form = Metadata.getInstance<Form>('formFreqProdGrupoPrato');
    const nmagrup = Metadata.getInstance<Select>('NMAGRUPGRPRAT');
    const nmgrupo = Metadata.getInstance<Select>('NMGRUPOPRATOPAE');
    const NMPRODUTO = Metadata.getInstance<Select>('NMPRODUTO');
    nmagrup.value = '';
    nmgrupo.value = '';
    NMPRODUTO.value = '';
    form.value.NRMINFREQGRPRATMES = null;
    form.value.NRMAXFREQGRPRATMES = null;
    form.value.NRMAXFREQGRPRATSEM = null;
    this.modalGrupoPrato.hide();
  }
  
  public newRow({ component }: IEventParam<Grid>) {
    const form = Metadata.getInstance<Form>('formFreqProdPer');
    if (form.validate()) this.gridEditController.addRow({ component });
  }
}
